//
// Original source/logic comes from the `utils.js` file of the Alpine.js project:
// https://github.com/alpinejs/alpine/blob/master/src/utils.js
//
import { __read, __spread } from "tslib";
export function transitionClasses(animationFrameRef, getComputedStyleRef, timeoutRef, el, classesDuring, classesStart, classesEnd, hook1, hook2) {
    transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, {
        start: function () {
            var _a;
            (_a = el.classList).add.apply(_a, __spread(classesStart));
        },
        during: function () {
            var _a;
            (_a = el.classList).add.apply(_a, __spread(classesDuring));
        },
        show: function () {
            hook1();
        },
        end: function () {
            var _a, _b;
            (_a = el.classList).remove.apply(_a, __spread(classesStart));
            (_b = el.classList).add.apply(_b, __spread(classesEnd));
        },
        hide: function () {
            hook2();
        },
        cleanup: function () {
            var _a, _b;
            (_a = el.classList).remove.apply(_a, __spread(classesDuring));
            (_b = el.classList).remove.apply(_b, __spread(classesEnd));
        }
    });
}
export function transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, stages) {
    stages.start();
    stages.during();
    animationFrameRef.nativeRequest(function () {
        // Note: Safari's transitionDuration property will list out comma separated
        // transition durations for every single transition property. Let's grab the
        // first one and call it a day.
        var transitionDuration = getComputedStyleRef.native(el).transitionDuration;
        var durationString = transitionDuration
            .replace(/,.*/, '')
            .replace('s', '');
        var duration = Number(durationString) * 1000;
        stages.show();
        animationFrameRef.nativeRequest(function () {
            stages.end();
            timeoutRef.nativeSet(function () {
                stages.hide();
                // Adding an "isConnected" check, in case the callback
                // removed the element from the DOM.
                if (el.isConnected) {
                    stages.cleanup();
                }
            }, duration);
        });
    });
}
//# sourceMappingURL=transition-classes.js.map