import { __decorate, __metadata } from 'tslib';
import { ViewContainerRef, TemplateRef, Input, Directive, NgModule } from '@angular/core';
import { NgIfContext, CommonModule } from '@angular/common';
import { AnimationFrameRef, GetComputedStyleRef, TimeoutRef } from 'ng-refs';

//
// Original source/logic comes from the `utils.js` file of the Alpine.js project:
// https://github.com/alpinejs/alpine/blob/master/src/utils.js
//
function transitionClasses(animationFrameRef, getComputedStyleRef, timeoutRef, el, classesDuring, classesStart, classesEnd, hook1, hook2) {
    transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, {
        start() {
            el.classList.add(...classesStart);
        },
        during() {
            el.classList.add(...classesDuring);
        },
        show() {
            hook1();
        },
        end() {
            el.classList.remove(...classesStart);
            el.classList.add(...classesEnd);
        },
        hide() {
            hook2();
        },
        cleanup() {
            el.classList.remove(...classesDuring);
            el.classList.remove(...classesEnd);
        }
    });
}
function transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, stages) {
    stages.start();
    stages.during();
    animationFrameRef.nativeRequest(() => {
        // Note: Safari's transitionDuration property will list out comma separated
        // transition durations for every single transition property. Let's grab the
        // first one and call it a day.
        const { transitionDuration } = getComputedStyleRef.native(el);
        const durationString = transitionDuration
            .replace(/,.*/, '')
            .replace('s', '');
        const duration = Number(durationString) * 1000;
        stages.show();
        animationFrameRef.nativeRequest(() => {
            stages.end();
            timeoutRef.nativeSet(() => {
                stages.hide();
                // Adding an "isConnected" check, in case the callback
                // removed the element from the DOM.
                if (el.isConnected) {
                    stages.cleanup();
                }
            }, duration);
        });
    });
}

const NOOP = () => void 0;
const ɵ0 = NOOP;
let NgTransitionDirective = class NgTransitionDirective {
    constructor(animationFrameRef, getComputedStyleRef, timeoutRef, _viewContainer, templateRef) {
        this.animationFrameRef = animationFrameRef;
        this.getComputedStyleRef = getComputedStyleRef;
        this.timeoutRef = timeoutRef;
        this._viewContainer = _viewContainer;
        this.templateRef = templateRef;
        this._context = new NgIfContext();
        this._init = false;
        this._thenTemplateRef = null;
        this._thenViewRef = null;
        this._ngTransitionEnter = [];
        this._ngTransitionEnterStart = [];
        this._ngTransitionEnterEnd = [];
        this._ngTransitionLeave = [];
        this._ngTransitionLeaveStart = [];
        this._ngTransitionLeaveEnd = [];
        this._thenTemplateRef = templateRef;
    }
    ngOnInit() {
        this._init = true;
        this._updateView();
    }
    set ngTransition(condition) {
        this._context.$implicit = this._context.ngIf = condition;
        this._updateView();
    }
    set ngTransitionEnter(ngTransitionEnter) {
        this._ngTransitionEnter = ngTransitionEnter.split(' ');
    }
    set ngTransitionEnterStart(ngTransitionEnterStart) {
        this._ngTransitionEnterStart = ngTransitionEnterStart.split(' ');
    }
    set ngTransitionEnterEnd(ngTransitionEnterEnd) {
        this._ngTransitionEnterEnd = ngTransitionEnterEnd.split(' ');
    }
    set ngTransitionLeave(ngTransitionLeave) {
        this._ngTransitionLeave = ngTransitionLeave.split(' ');
    }
    set ngTransitionLeaveStart(ngTransitionLeaveStart) {
        this._ngTransitionLeaveStart = ngTransitionLeaveStart.split(' ');
    }
    set ngTransitionLeaveEnd(ngTransitionLeaveEnd) {
        this._ngTransitionLeaveEnd = ngTransitionLeaveEnd.split(' ');
    }
    _updateView() {
        if (!this._init) {
            return;
        }
        if (this._context.$implicit) {
            this.show();
        }
        else {
            this.hide();
        }
    }
    show() {
        if (!this._thenTemplateRef) {
            return;
        }
        if (this._thenViewRef) {
            this._thenViewRef.destroy();
            this._thenViewRef = null;
        }
        this._thenViewRef = this._viewContainer.createEmbeddedView(this._thenTemplateRef, this._context);
        transitionClasses(this.animationFrameRef, this.getComputedStyleRef, this.timeoutRef, this._thenViewRef.rootNodes[0], this._ngTransitionEnter, this._ngTransitionEnterStart, this._ngTransitionEnterEnd, NOOP, NOOP);
    }
    hide() {
        if (!this._thenViewRef) {
            return;
        }
        transitionClasses(this.animationFrameRef, this.getComputedStyleRef, this.timeoutRef, this._thenViewRef.rootNodes[0], this._ngTransitionLeave, this._ngTransitionLeaveStart, this._ngTransitionLeaveEnd, NOOP, () => this._viewContainer.clear());
    }
};
NgTransitionDirective.ctorParameters = () => [
    { type: AnimationFrameRef },
    { type: GetComputedStyleRef },
    { type: TimeoutRef },
    { type: ViewContainerRef },
    { type: TemplateRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], NgTransitionDirective.prototype, "ngTransition", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionEnter", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionEnterStart", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionEnterEnd", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionLeave", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionLeaveStart", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], NgTransitionDirective.prototype, "ngTransitionLeaveEnd", null);
NgTransitionDirective = __decorate([
    Directive({
        selector: '[ngTransition]'
    }),
    __metadata("design:paramtypes", [AnimationFrameRef,
        GetComputedStyleRef,
        TimeoutRef,
        ViewContainerRef,
        TemplateRef])
], NgTransitionDirective);

let NgTransitionModule = class NgTransitionModule {
};
NgTransitionModule = __decorate([
    NgModule({
        imports: [CommonModule],
        declarations: [NgTransitionDirective],
        providers: [AnimationFrameRef, GetComputedStyleRef, TimeoutRef],
        exports: [NgTransitionDirective]
    })
], NgTransitionModule);

/**
 * Generated bundle index. Do not edit.
 */

export { NgTransitionModule, NgTransitionDirective as ɵa };
//# sourceMappingURL=ng-transition.js.map
