import { __spread, __decorate, __metadata } from 'tslib';
import { ViewContainerRef, TemplateRef, Input, Directive, NgModule } from '@angular/core';
import { NgIfContext, CommonModule } from '@angular/common';
import { AnimationFrameRef, GetComputedStyleRef, TimeoutRef } from 'ng-refs';

//
function transitionClasses(animationFrameRef, getComputedStyleRef, timeoutRef, el, classesDuring, classesStart, classesEnd, hook1, hook2) {
    transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, {
        start: function () {
            var _a;
            (_a = el.classList).add.apply(_a, __spread(classesStart));
        },
        during: function () {
            var _a;
            (_a = el.classList).add.apply(_a, __spread(classesDuring));
        },
        show: function () {
            hook1();
        },
        end: function () {
            var _a, _b;
            (_a = el.classList).remove.apply(_a, __spread(classesStart));
            (_b = el.classList).add.apply(_b, __spread(classesEnd));
        },
        hide: function () {
            hook2();
        },
        cleanup: function () {
            var _a, _b;
            (_a = el.classList).remove.apply(_a, __spread(classesDuring));
            (_b = el.classList).remove.apply(_b, __spread(classesEnd));
        }
    });
}
function transition(animationFrameRef, getComputedStyleRef, timeoutRef, el, stages) {
    stages.start();
    stages.during();
    animationFrameRef.nativeRequest(function () {
        // Note: Safari's transitionDuration property will list out comma separated
        // transition durations for every single transition property. Let's grab the
        // first one and call it a day.
        var transitionDuration = getComputedStyleRef.native(el).transitionDuration;
        var durationString = transitionDuration
            .replace(/,.*/, '')
            .replace('s', '');
        var duration = Number(durationString) * 1000;
        stages.show();
        animationFrameRef.nativeRequest(function () {
            stages.end();
            timeoutRef.nativeSet(function () {
                stages.hide();
                // Adding an "isConnected" check, in case the callback
                // removed the element from the DOM.
                if (el.isConnected) {
                    stages.cleanup();
                }
            }, duration);
        });
    });
}

var NOOP = function () { return void 0; };
var ɵ0 = NOOP;
var NgTransitionDirective = /** @class */ (function () {
    function NgTransitionDirective(animationFrameRef, getComputedStyleRef, timeoutRef, _viewContainer, templateRef) {
        this.animationFrameRef = animationFrameRef;
        this.getComputedStyleRef = getComputedStyleRef;
        this.timeoutRef = timeoutRef;
        this._viewContainer = _viewContainer;
        this.templateRef = templateRef;
        this._context = new NgIfContext();
        this._init = false;
        this._thenTemplateRef = null;
        this._thenViewRef = null;
        this._ngTransitionEnter = [];
        this._ngTransitionEnterStart = [];
        this._ngTransitionEnterEnd = [];
        this._ngTransitionLeave = [];
        this._ngTransitionLeaveStart = [];
        this._ngTransitionLeaveEnd = [];
        this._thenTemplateRef = templateRef;
    }
    NgTransitionDirective.prototype.ngOnInit = function () {
        this._init = true;
        this._updateView();
    };
    Object.defineProperty(NgTransitionDirective.prototype, "ngTransition", {
        set: function (condition) {
            this._context.$implicit = this._context.ngIf = condition;
            this._updateView();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgTransitionDirective.prototype, "ngTransitionEnter", {
        set: function (ngTransitionEnter) {
            this._ngTransitionEnter = ngTransitionEnter.split(' ');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgTransitionDirective.prototype, "ngTransitionEnterStart", {
        set: function (ngTransitionEnterStart) {
            this._ngTransitionEnterStart = ngTransitionEnterStart.split(' ');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgTransitionDirective.prototype, "ngTransitionEnterEnd", {
        set: function (ngTransitionEnterEnd) {
            this._ngTransitionEnterEnd = ngTransitionEnterEnd.split(' ');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgTransitionDirective.prototype, "ngTransitionLeave", {
        set: function (ngTransitionLeave) {
            this._ngTransitionLeave = ngTransitionLeave.split(' ');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgTransitionDirective.prototype, "ngTransitionLeaveStart", {
        set: function (ngTransitionLeaveStart) {
            this._ngTransitionLeaveStart = ngTransitionLeaveStart.split(' ');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgTransitionDirective.prototype, "ngTransitionLeaveEnd", {
        set: function (ngTransitionLeaveEnd) {
            this._ngTransitionLeaveEnd = ngTransitionLeaveEnd.split(' ');
        },
        enumerable: true,
        configurable: true
    });
    NgTransitionDirective.prototype._updateView = function () {
        if (!this._init) {
            return;
        }
        if (this._context.$implicit) {
            this.show();
        }
        else {
            this.hide();
        }
    };
    NgTransitionDirective.prototype.show = function () {
        if (!this._thenTemplateRef) {
            return;
        }
        if (this._thenViewRef) {
            this._thenViewRef.destroy();
            this._thenViewRef = null;
        }
        this._thenViewRef = this._viewContainer.createEmbeddedView(this._thenTemplateRef, this._context);
        transitionClasses(this.animationFrameRef, this.getComputedStyleRef, this.timeoutRef, this._thenViewRef.rootNodes[0], this._ngTransitionEnter, this._ngTransitionEnterStart, this._ngTransitionEnterEnd, NOOP, NOOP);
    };
    NgTransitionDirective.prototype.hide = function () {
        var _this = this;
        if (!this._thenViewRef) {
            return;
        }
        transitionClasses(this.animationFrameRef, this.getComputedStyleRef, this.timeoutRef, this._thenViewRef.rootNodes[0], this._ngTransitionLeave, this._ngTransitionLeaveStart, this._ngTransitionLeaveEnd, NOOP, function () { return _this._viewContainer.clear(); });
    };
    NgTransitionDirective.ctorParameters = function () { return [
        { type: AnimationFrameRef },
        { type: GetComputedStyleRef },
        { type: TimeoutRef },
        { type: ViewContainerRef },
        { type: TemplateRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], NgTransitionDirective.prototype, "ngTransition", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgTransitionDirective.prototype, "ngTransitionEnter", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgTransitionDirective.prototype, "ngTransitionEnterStart", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgTransitionDirective.prototype, "ngTransitionEnterEnd", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgTransitionDirective.prototype, "ngTransitionLeave", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgTransitionDirective.prototype, "ngTransitionLeaveStart", null);
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], NgTransitionDirective.prototype, "ngTransitionLeaveEnd", null);
    NgTransitionDirective = __decorate([
        Directive({
            selector: '[ngTransition]'
        }),
        __metadata("design:paramtypes", [AnimationFrameRef,
            GetComputedStyleRef,
            TimeoutRef,
            ViewContainerRef,
            TemplateRef])
    ], NgTransitionDirective);
    return NgTransitionDirective;
}());

var NgTransitionModule = /** @class */ (function () {
    function NgTransitionModule() {
    }
    NgTransitionModule = __decorate([
        NgModule({
            imports: [CommonModule],
            declarations: [NgTransitionDirective],
            providers: [AnimationFrameRef, GetComputedStyleRef, TimeoutRef],
            exports: [NgTransitionDirective]
        })
    ], NgTransitionModule);
    return NgTransitionModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { NgTransitionModule, NgTransitionDirective as ɵa };
//# sourceMappingURL=ng-transition.js.map
